#include <liquid/liquid.h>

#include "freq_est.h"

// Fits a line y' = m*x' + t to the given points (x, y), where x is the
// zero-based index in the array.
static void linear_regression(size_t nx, const float *y, float *m, float *t)
{
	float mean_x = (nx-1) / 2.0f;
	float mean_y = 0.0f;
	for(unsigned int j = 0; j < nx; j++) {
		mean_y += y[j];
	}
	mean_y /= nx;

	float numerator = 0.0f;
	float denominator = 0.0f;
	for(unsigned int j = 0; j < nx; j++) {
		float delta_index = j - mean_x;
		numerator += delta_index * (y[j] - mean_y);
		denominator += delta_index*delta_index;
	}

	*m = numerator / denominator;
	*t = mean_y - (*m) * mean_x;
}


float freq_est_unknown_bpsk(const float complex *recv, size_t n, float *final_phase)
{
	float phases[n];

	// square the symbols and calculate phase
	for(size_t i = 0; i < n; i++) {
		float complex s = recv[i];
		phases[i] = cargf(s * s);
	}

	liquid_unwrap_phase(phases, n);

	float freq, phi0;
	linear_regression(n, phases, &freq, &phi0);

	// outputs must be divided by 2 because squaring the symbols doubles their phase.
	if(final_phase) {
		*final_phase = (phi0 + n * freq) / 2;
	}

	return freq/2;
}


float freq_est_known_symbols(const float complex *recv, const float complex *ref, size_t n, float *final_phase)
{
	float complex symbols[n];

	// remove the data influence from the symbols
	for(size_t i = 0; i < n; i++) {
		symbols[i] = conjf(ref[i]) * recv[i];
	}

	return freq_est_data_free(symbols, n, final_phase);
}

float freq_est_data_free(const float complex *symbols, size_t n, float *final_phase)
{
	float phases[n];

	for(size_t i = 0; i < n; i++) {
		phases[i] = cargf(symbols[i]);
	}

	liquid_unwrap_phase(phases, n);

	float freq, phi0;
	linear_regression(n, phases, &freq, &phi0);

	if(final_phase) {
		*final_phase = phi0 + n * freq;
	}

	return freq;
}
